@@ -13,6 +13,7 @@
 #include "nest/cmds.h"
 #include "lib/string.h"
 #include "lib/resource.h"
+#include "filter/filter.h"
 
 extern int shutting_down;
 extern int configuring;
@@ -90,3 +91,20 @@ cmd_show_memory(void)
   print_size("Total:", rmemsize(&root_pool));
   cli_msg(0, "");
 }
+
+void
+cmd_eval(struct f_inst *expr)
+{
+  struct f_val v = f_eval(expr, this_cli->parser_pool);
+
+  if (v.type == T_RETURN)
+    {
+      cli_msg(8008, "runtime error");
+      return;
+    }
+
+  buffer buf;
+  LOG_BUFFER_INIT(buf);
+  val_format(v, &buf);
+  cli_msg(23, "%s", buf.start);
+}
