@@ -14,19 +14,26 @@
 struct password_item *last_password_item = NULL;
 
 struct password_item *
-password_find(list *l)
+password_find(list *l, int first_fit)
 {
   struct password_item *pi;
+  struct password_item *pf = NULL;
 
   if (l)
   {
     WALK_LIST(pi, *l)
     {
-      if ((pi->genfrom < now) && (pi->gento > now))
-        return pi;
+      if ((pi->genfrom < now_real) && (pi->gento > now_real))
+      {
+	if (first_fit)
+	  return pi;
+
+	if (!pf || pf->genfrom < pi->genfrom)
+	  pf = pi;
+      }
     }
   }
-  return NULL;
+  return pf;
 }
 
 void password_cpy(char *dst, char *src, int size)
